@@ -1,6 +1,7 @@
 import json
 import logging
-import typing
+import threading
+from typing import cast, Mapping, Optional, Generator, List, Union
 from unittest import mock
 
 import boto3
@@ -18,11 +19,18 @@ logger = logging.getLogger(__name__)
 
 
 class AWSMetadata:
-    def __init__(self, receipt):
-        self._receipt = receipt
+    """
+    AWS specific metadata for a Message
+    """
+
+    def __init__(self, receipt: str):
+        self._receipt: str = receipt
 
     @property
-    def receipt(self):
+    def receipt(self) -> str:
+        """
+        AWS receipt identifier
+        """
         return self._receipt
 
     def __eq__(self, o: object) -> bool:
@@ -45,23 +53,29 @@ class AWSMetadata:
 
 class AWSSNSPublisherBackend(HedwigPublisherBaseBackend):
     def __init__(self):
-        config = Config(connect_timeout=settings.AWS_CONNECT_TIMEOUT_S, read_timeout=settings.AWS_READ_TIMEOUT_S)
-        self.sns_client = boto3.client(
-            'sns',
-            region_name=settings.AWS_REGION,
-            aws_access_key_id=settings.AWS_ACCESS_KEY,
-            aws_secret_access_key=settings.AWS_SECRET_KEY,
-            aws_session_token=settings.AWS_SESSION_TOKEN,
-            endpoint_url=settings.AWS_ENDPOINT_SNS,
-            config=config,
-        )
+        self._sns_client = None
+
+    @property
+    def sns_client(self):
+        if self._sns_client is None:
+            config = Config(connect_timeout=settings.AWS_CONNECT_TIMEOUT_S, read_timeout=settings.AWS_READ_TIMEOUT_S)
+            self._sns_client = boto3.client(
+                'sns',
+                region_name=settings.AWS_REGION,
+                aws_access_key_id=settings.AWS_ACCESS_KEY,
+                aws_secret_access_key=settings.AWS_SECRET_KEY,
+                aws_session_token=settings.AWS_SESSION_TOKEN,
+                endpoint_url=settings.AWS_ENDPOINT_SNS,
+                config=config,
+            )
+        return self._sns_client
 
     @staticmethod
     def _get_sns_topic(message: Message) -> str:
         return f'arn:aws:sns:{settings.AWS_REGION}:{settings.AWS_ACCOUNT_ID}:hedwig-{message.topic}'
 
     @retry(stop_max_attempt_number=3, stop_max_delay=3000)
-    def _publish_over_sns(self, topic: str, message_json: str, message_attributes: dict) -> str:
+    def _publish_over_sns(self, topic: str, message_json: str, message_attributes: Mapping) -> str:
         # transform (http://boto.cloudhackers.com/en/latest/ref/sns.html#boto.sns.SNSConnection.publish)
         message_attributes = {k: {'DataType': 'String', 'StringValue': str(v)} for k, v in message_attributes.items()}
         response = self.sns_client.publish(TopicArn=topic, Message=message_json, MessageAttributes=message_attributes)
@@ -73,7 +87,7 @@ class AWSSNSPublisherBackend(HedwigPublisherBaseBackend):
         sqs_message.receipt_handle = 'test-receipt'
         return sqs_message
 
-    def _publish(self, message: Message, payload: str, headers: typing.Optional[typing.Mapping] = None) -> str:
+    def _publish(self, message: Message, payload: str, headers: Optional[Mapping] = None) -> str:
         topic = self._get_sns_topic(message)
         return self._publish_over_sns(topic, payload, headers)
 
@@ -82,28 +96,49 @@ class AWSSQSConsumerBackend(HedwigConsumerBaseBackend):
     WAIT_TIME_SECONDS = 20
 
     def __init__(self, dlq=False):
-        self.sqs_resource = boto3.resource(
-            'sqs',
-            region_name=settings.AWS_REGION,
-            aws_access_key_id=settings.AWS_ACCESS_KEY,
-            aws_secret_access_key=settings.AWS_SECRET_KEY,
-            aws_session_token=settings.AWS_SESSION_TOKEN,
-            endpoint_url=settings.AWS_ENDPOINT_SQS,
-        )
-        self.sqs_client = boto3.client(
-            'sqs',
-            region_name=settings.AWS_REGION,
-            aws_access_key_id=settings.AWS_ACCESS_KEY,
-            aws_secret_access_key=settings.AWS_SECRET_KEY,
-            aws_session_token=settings.AWS_SESSION_TOKEN,
-            endpoint_url=settings.AWS_ENDPOINT_SQS,
-        )
+        self._sqs_resource = None
+        self._sqs_client = None
         self.queue_name = f'HEDWIG-{settings.HEDWIG_QUEUE}{"-DLQ" if dlq else ""}'
 
+    @property
+    def sqs_resource(self):
+        if self._sqs_resource is None:
+            self._sqs_resource = boto3.resource(
+                'sqs',
+                region_name=settings.AWS_REGION,
+                aws_access_key_id=settings.AWS_ACCESS_KEY,
+                aws_secret_access_key=settings.AWS_SECRET_KEY,
+                aws_session_token=settings.AWS_SESSION_TOKEN,
+                endpoint_url=settings.AWS_ENDPOINT_SQS,
+            )
+        return self._sqs_resource
+
+    @property
+    def sqs_client(self):
+        if self._sqs_client is None:
+            self._sqs_client = boto3.client(
+                'sqs',
+                region_name=settings.AWS_REGION,
+                aws_access_key_id=settings.AWS_ACCESS_KEY,
+                aws_secret_access_key=settings.AWS_SECRET_KEY,
+                aws_session_token=settings.AWS_SESSION_TOKEN,
+                endpoint_url=settings.AWS_ENDPOINT_SQS,
+            )
+        return self._sqs_client
+
     def _get_queue(self):
         return self.sqs_resource.get_queue_by_name(QueueName=self.queue_name)
 
-    def pull_messages(self, num_messages: int = 1, visibility_timeout: int = None) -> typing.List:
+    def pull_messages(
+        self, num_messages: int = 10, visibility_timeout: int = None, shutdown_event: Optional[threading.Event] = None
+    ) -> Union[Generator, List]:
+        """
+
+        :param shutdown_event: Unused for this backend
+        :param num_messages:
+        :param visibility_timeout:
+        :return:
+        """
         params = {
             'MaxNumberOfMessages': num_messages,
             'WaitTimeSeconds': self.WAIT_TIME_SECONDS,
@@ -118,9 +153,13 @@ class AWSSQSConsumerBackend(HedwigConsumerBaseBackend):
         receipt = queue_message.receipt_handle
         self.message_handler(message_json, AWSMetadata(receipt))
 
-    def delete_message(self, queue_message) -> None:
+    def ack_message(self, queue_message) -> None:
         queue_message.delete()
 
+    def nack_message(self, queue_message) -> None:
+        # let visibility timeout take care of it
+        pass
+
     def extend_visibility_timeout(self, visibility_timeout_s: int, metadata: AWSMetadata) -> None:
         """
         Extends visibility timeout of a message on a given priority queue for long running tasks.
@@ -145,6 +184,8 @@ class AWSSQSConsumerBackend(HedwigConsumerBaseBackend):
         logging.info("Re-queueing messages from {} to {}".format(dead_letter_queue.url, sqs_queue.url))
         while True:
             queue_messages = self.pull_messages(num_messages=num_messages, visibility_timeout=visibility_timeout)
+            queue_messages = cast(list, queue_messages)
+
             if not queue_messages:
                 break
 
@@ -185,10 +226,15 @@ class AWSSNSConsumerBackend(HedwigConsumerBaseBackend):
     def requeue_dead_letter(self, num_messages: int = 10, visibility_timeout: int = None) -> None:
         raise RuntimeError("invalid operation for backend")
 
-    def pull_messages(self, num_messages: int = 1, visibility_timeout: int = None) -> typing.List:
+    def pull_messages(
+        self, num_messages: int = 10, visibility_timeout: int = None, shutdown_event: Optional[threading.Event] = None
+    ) -> Union[Generator, List]:
         raise RuntimeError("invalid operation for backend")
 
-    def delete_message(self, queue_message) -> None:
+    def ack_message(self, queue_message) -> None:
+        raise RuntimeError("invalid operation for backend")
+
+    def nack_message(self, queue_message) -> None:
         raise RuntimeError("invalid operation for backend")
 
     def extend_visibility_timeout(self, visibility_timeout_s: int, metadata) -> None:
